#include <iostream>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <glob.h>

using namespace std;

#include "clases.hc"

#define OPC1	(GLOB_TILDE|GLOB_NOCHECK)
#define OPC2	(OPC1|GLOB_APPEND)
#define MAX_N	1024

enum exne {EXIT, NOEXIT};
static void error_exit(const char *s, enum exne en)
{
	char err[1024];
	strerror_r(errno, err, sizeof err);
	cerr<<s<<':'<<err<<'\n';
	if(en==EXIT) exit(-1);
}
static void glob_error(int err)
{
	cerr<<"glob: ";
	switch(err) {
	case GLOB_NOSPACE: cerr<<"nospace\n!"; break;
	case GLOB_ABORTED: cerr<<"aborted\n!"; break;
	case GLOB_NOMATCH: cerr<<"nomatch\n"; break;
	default: cerr<<"oops\n!";
	}
}
Cd::Cd(char *dir)
{
	NEW(Cd::dir, char[strlen(dir)+1]);
	strcpy(Cd::dir, dir);
	DELETE(dir);
}
int Cd::exec()
{
	glob_t g;
	int r;
	if((r=glob(dir, OPC1, NULL, &g)))
		glob_error(r);
	if((r=chdir(g.gl_pathv[0])))
		error_exit(g.gl_pathv[0], NOEXIT);
	globfree(&g);
	return r;
}
char *Comando::path(char *p)
{
	char *tmp;
	NEW(tmp, char[1024]);
	if(strchr(p, '/')!=0) {
		strcpy(tmp, p);
		return tmp;
	}
	char env[1024];
	strcpy(env, getenv("PATH"));
	char *pc=strtok(env, ":");
	do {
		strcat(strcat(strcpy(tmp, pc), "/"), p);
		if(access(tmp, X_OK)==0)
			return tmp;
	} while((pc=strtok(0, ":"))!=0);
	return p;
}
Comando::~Comando()
{
	DELETE(prog);
	for(int i=1; args[i]; i++)
		DELETE(args[i]);
	DELETE(args);
}
int Comando::exec()
{
	prog=path(prog);
	if(FORK("Comando")==0) {
		glob_t pg;
		int r;
		if((r=glob(args[0], OPC1, NULL, &pg)))
			glob_error(r);
		for(int i=1; args[i]; i++)
			if((r=glob(args[i], OPC2, NULL, &pg)))
				glob_error(r);
		execv(prog, pg.gl_pathv);
		error_exit(prog, EXIT);
	}
	int status;
	wait(&status);
	return status;
}
int Pipe::exec()
{
	int pip[2], status;
	if(fork()!=0)
		wait(&status);
	else {
		if(pipe(pip)) error_exit("pipe", EXIT);
		if(FORK("Pipe 2")!=0) {
			dup2(pip[1], 1);
			close(pip[0]); close(pip[1]);
			exit(i->exec());
		}
		else {
			dup2(pip[0], 0);
			close(pip[0]); close(pip[1]);
			exit(d->exec());
		}
	}
	return status;
}
Redir::Redir(Accion *p, char *arch, enum modoRedir mr)
{
	Redir::p=p;
	NEW(Redir::arch, char[strlen(arch)+1]);
	strcpy(Redir::arch, arch);
	DELETE(arch);
	Redir::mr=mr;
}
int Redir::exec()
{
	int status;
	if(fork()!=0) {
		wait(&status);
		if(WIFEXITED(status)) status=WEXITSTATUS(status);
	}
	else {
		int md=mr==Inp? O_RDONLY: O_WRONLY|O_CREAT|
					(mr==OutApp? O_APPEND: O_TRUNC);
		int f=open(arch, md, 0666);
		if(f<0) error_exit(arch, EXIT);
		dup2(f, mr==Inp? 0: 1);
		exit(p->exec());
	}
	return status;
}
int And::exec()
{
	int status;
	status=i->exec();
	if(WIFEXITED(status) && !WEXITSTATUS(status))
		status=d->exec();
	return status;
}
int Or::exec()
{
	int status;
	// COMPLETAR
}
int Secuencia::exec()
{
	int status;
	// COMPLETAR
}
int Subshell::exec()
{
	int status;
	// COMPLETAR
}
