#ifndef __CLASES_H
#define __CLASES_H

#include <iostream>
using namespace std;

#ifdef _EFENCE
	#define NEW(p, q)	{ p=new q; cerr<<"new "<<(unsigned*)p<< \
						" ("<<__FILE__<<','<<__LINE__<<")\n"; }
	#define DELETE(p)	{ cerr<<"delete "<<(unsigned*)p<< \
						" ("<<__FILE__<<','<<__LINE__<<")\n"; delete p; }
	#define DELETEA(p)	{ cerr<<"deletea "<<(unsigned*)p<< \
						" ("<<__FILE__<<','<<__LINE__<<")\n"; delete[] p; }
	#define FORK(x)		({ int i=fork(); cerr<<x<<" "<<i<<'\n'; i; })
#else
	#define NEW(p, q)	p=new q
	#define DELETE(p)	delete p
	#define DELETEA(p)	delete[] p
	#define FORK(x)		fork()
#endif

class Accion {
public:
	virtual ~Accion() {}
	virtual int exec()=0;
};
class Pwd: public Accion {
public:
	int exec() {
		cout<<get_current_dir_name()<<'\n';
		return 0;
	}
};
class Cd: public Accion {
	char *dir;
public:
	Cd(char *dir);
	~Cd() { DELETEA(dir); }
	int exec();
};
class Logout: public Accion {
public:
	int exec() { exit(0); return 0; }
};
class Echo: public Accion {
	char *str;
public:
	Echo(char *str): str(str) {}
	int exec() { cout<<str<<'\n'; return 0; }
};
class Comando: public Accion {
	char *prog, **args;
	static char *path(char *);
public:
	Comando(char *prog, char **args): prog(prog), args(args) {}
	~Comando();
	int exec();
};
class Pipe: public Accion {
	Accion *i, *d;
public:
	Pipe(Accion *i, Accion *d): i(i), d(d) {}
	~Pipe() { DELETE(i); DELETE(d); }
	int exec();	
};
class Redir: public Accion {
	Accion *p;
	char *arch;
public:
	enum modoRedir { Inp, OutApp, OutTrunc };
private:
	enum modoRedir mr;
public:
	Redir(Accion *p, char *arch, enum modoRedir mr);
	~Redir() { DELETE(p); DELETE(arch); }
	int exec();	
};
class And: public Accion {
	Accion *i, *d;
public:
	And(Accion *i, Accion *d): i(i), d(d) {}
	~And() { DELETE(i); DELETE(d); }
	int exec();
};
class Or: public Accion {
	Accion *i, *d;
public:
	Or(Accion *i, Accion *d): i(i), d(d) {}
	~Or() { DELETE(i); DELETE(d); }
	int exec();
};
class Secuencia: public Accion {
	Accion *i, *d;
public:
	Secuencia(Accion *i, Accion *d): i(i), d(d) {}
	~Secuencia() { DELETE(i); DELETE(d); }
	int exec();	
};
class Subshell: public Accion {
	Accion *a;
public:
	Subshell(Accion *a): a(a) {}
	~Subshell() { DELETE(a); }
	int exec();	
};
#endif
